"""Weather site Module."""
from pysolcast.base import PySolcast


class WeatherSite(PySolcast):
    """Class for interacting with weather sites.

    Refer to https://docs.solcast.com.au/#weather-site for more information.
    """

    base_uri = 'weather_sites'

    def get_forecasts(self) -> dict:
        """Get forecasts data for site.

        :returns: forecasts
        :raises ValidationError:
        :raises SiteError:
        """
        endpoint = 'forecasts'
        return self._get_data(self._create_uri(self.base_uri, endpoint))

    def get_estimated_actuals(self) -> dict:
        """Get estimated actuals data for site.

        :returns: estimated_actuals
        :raises ValidationError:
        :raises SiteError:
        """
        endpoint = 'estimated_actuals'
        return self._get_data(self._create_uri(self.base_uri, endpoint))



"""Tests for World class."""

import responses
import pytest
from pysolcast.world import World
from pysolcast.exceptions import ValidationError, SiteError, RateLimitExceeded

BASE_URL = 'https://api.solcast.com.au'
WORLD_URI = '/world_radiation/'


def test_World():
    """Test creating object."""
    # Arrange
    api_key = '12345'

    # Act
    site = World(api_key)

    # Assert
    assert isinstance(site, World)


@responses.activate
def test_get_forecasts_200():
    """Test get_forecasts with 200 status code."""
    # Arrange
    api_key = '12345'
    endpoint = 'forecasts'
    latitude = '-35.123'
    longitude = '149.123'
    expected_url = f'{BASE_URL}{WORLD_URI}{endpoint}'

    forecast_response = {
        "forecasts": [
            {
                "ghi": 690,
                "ghi90": 802,
                "ghi10": 537,
                "period_end": "2017-01-30T05:00:00.0000000Z",
                "period": "PT30M"
            },
            {
                "ghi": 422,
                "ghi90": 707,
                "ghi10": 141,
                "period_end": "2017-01-30T05:30:00.0000000Z",
                "period": "PT30M"
            }
        ]
    }

    responses.add(
        responses.GET,
        expected_url,
        json=forecast_response,
        status=200,
        content_type='applicaiton/json'
    )

    # Act
    site = World(api_key)
    forecasts = site.get_forecasts(latitude, longitude)

    # Assert
    assert isinstance(forecasts, dict)
    assert len(forecasts['forecasts']) == 2


@responses.activate
def test_get_forecasts_400():
    """Test get_forecasts with 400 status code."""
    # Arrange
    api_key = '12345'
    endpoint = 'forecasts'
    latitude = '-35.123'
    longitude = '149.123'
    expected_url = f'{BASE_URL}{WORLD_URI}{endpoint}'

    responses.add(
        responses.GET, expected_url, status=400
    )

    # Act
    site = World(api_key)
    with pytest.raises(ValidationError):
        site.get_forecasts(latitude, longitude)

    # Assert


@responses.activate
def test_get_forecasts_404():
    """Test get_forecasts with 400 status code."""
    # Arrange
    api_key = '12345'
    endpoint = 'forecasts'
    latitude = '-35.123'
    longitude = '149.123'
    expected_url = f'{BASE_URL}{WORLD_URI}{endpoint}'

    responses.add(
        responses.GET, expected_url, status=404
    )

    # Act
    site = World(api_key)
    with pytest.raises(SiteError):
        site.get_forecasts(latitude, longitude)

    # Assert


@responses.activate
def test_get_forecasts_429():
    """Test get_forecasts with 429 status code."""
    # Arrange
    api_key = '12345'
    endpoint = 'forecasts'
    latitude = '-35.123'
    longitude = '149.123'
    expected_url = f'{BASE_URL}{WORLD_URI}{endpoint}'
    headers = {
        'x-rate-limit-reset': '155555555'
    }

    responses.add(
        responses.GET, expected_url, status=429, adding_headers=headers)

    # Act
    site = World(api_key)
    with pytest.raises(RateLimitExceeded):
        site.get_forecasts(latitude, longitude)

    # Assert


@responses.activate
def test_get_estimated_actuals_200():
    """Test get_estimated_actuals with 200 status code."""
    # Arrange
    api_key = '12345'
    endpoint = 'estimated_actuals'
    latitude = '-35.123'
    longitude = '149.123'
    expected_url = f'{BASE_URL}{WORLD_URI}{endpoint}'

    estimated_actual_response = {
        "estimated_actuals": [
            {
                "ghi": 640,
                "ebh": 516,
                "dni": 803,
                "dhi": 124,
                "cloud_opacity": 0,
                "period_end": "2017-01-29T23:00:00.0000000Z",
                "period": "PT30M"
            },
            {
                "ghi": 543,
                "ebh": 430,
                "dni": 769,
                "dhi": 113,
                "cloud_opacity": 0,
                "period_end": "2017-01-29T22:30:00.0000000Z",
                "period": "PT30M"
            }
        ]
    }

    responses.add(
        responses.GET,
        expected_url,
        json=estimated_actual_response,
        status=200,
        content_type='applicaiton/json'
    )

    # Act
    site = World(api_key)
    estimated_actuals = site.get_estimated_actuals(latitude, longitude)

    # Assert
    assert isinstance(estimated_actuals, dict)
    assert len(estimated_actuals['estimated_actuals']) == 2


@responses.activate
def test_get_estimated_actuals_400():
    """Test get_estimated_actuals with 400 status code."""
    # Arrange
    api_key = '12345'
    endpoint = 'estimated_actuals'
    latitude = '-35.123'
    longitude = '149.123'
    expected_url = f'{BASE_URL}{WORLD_URI}{endpoint}'

    responses.add(
        responses.GET, expected_url, status=400
    )

    # Act
    site = World(api_key)
    with pytest.raises(ValidationError):
        site.get_estimated_actuals(latitude, longitude)

    # Assert


@responses.activate
def test_get_estimated_actuals_404():
    """Test get_estimated_actuals with 404 status code."""
    # Arrange
    api_key = '12345'
    endpoint = 'estimated_actuals'
    latitude = '-35.123'
    longitude = '149.123'
    expected_url = f'{BASE_URL}{WORLD_URI}{endpoint}'

    responses.add(
        responses.GET, expected_url, status=404
    )

    # Act
    site = World(api_key)
    with pytest.raises(SiteError):
        site.get_estimated_actuals(latitude, longitude)

    # Assert


@responses.activate
def test_get_estimated_actuals_429():
    """Test get_estimated_actuals with 429 status code."""
    # Arrange
    api_key = '12345'
    endpoint = 'estimated_actuals'
    latitude = '-35.123'
    longitude = '149.123'
    expected_url = f'{BASE_URL}{WORLD_URI}{endpoint}'
    headers = {
        'x-rate-limit-reset': '155555555'
    }
    responses.add(
        responses.GET, expected_url, status=429, adding_headers=headers
    )

    # Act
    site = World(api_key)
    with pytest.raises(RateLimitExceeded):
        site.get_estimated_actuals(latitude, longitude)

    # Assert



"""Exceptions Module."""


class ValidationError(Exception):  # pylint: disable=missing-class-docstring
    """Data validation error."""


class SiteError(Exception):  # pylint: disable=missing-class-docstring
    """Site Error.

    Site is not found or not accessable.
    """


class RateLimitExceeded(Exception):  # pylint: disable=missing-class-docstring
    """Rate limit exceeded."""



"""Tests for base module."""

import json
import responses
from requests.exceptions import ConnectTimeout
import pytest
from pysolcast.base import PySolcast


BASE_URL = 'https://api.solcast.com.au'


def test_Solcast_init():
    """Test creating Solcast object."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    #endpoint = '/blah'

    # Act
    obj = PySolcast(api_key, resource_id)

    # Assert
    assert isinstance(obj, PySolcast)


@responses.activate
def test_get_data_200():
    """Test get_data."""
    # Arrange

    api_key = '12345'
    resource_id = '1234-1234'
    uri = '/blah'
    expected_url = f'{BASE_URL}{uri}'
    expected_response = """{}"""

    responses.add(
        responses.GET,
        expected_url,
        body=expected_response,
        status=200,
        content_type='applicaiton/json'
    )

    # Act
    obj = PySolcast(api_key, resource_id)
    response = obj._get_data(uri)  # pylint: disable=protected-access

    # Assert
    assert response == json.loads(expected_response)
    assert responses.calls[0].request.headers['Authorization'] == 'Basic MTIzNDU6'


@responses.activate
def test_get_data_timeout():
    """Test get_data with timeout."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    uri = '/blah'
    expected_url = f'{BASE_URL}{uri}'

    responses.add(
        responses.GET,
        expected_url,
        body=ConnectTimeout(),
        status=200,
        content_type='applicaiton/json'
    )

    # Act
    obj = PySolcast(api_key, resource_id)
    with pytest.raises(ConnectTimeout):
        obj._get_data(uri)  # pylint: disable=protected-access

    # Assert
    assert responses.calls[0].request.headers['Authorization'] == 'Basic MTIzNDU6'


@responses.activate
def test_post_data():
    """Test post_measurements with 200 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = '/measurements'
    expected_url = f'{BASE_URL}{endpoint}'

    measurement_single = """{
        "measurement": {
            "period_end": "2018-02-02T03:30:00.0000000Z",
            "period": "PT5M",
            "total_power": 1.23456
        }
    }"""

    responses.add(
        responses.POST,
        expected_url,
        body=measurement_single,
        status=200,
        content_type='applicaiton/json'
    )

    # Act
    obj = PySolcast(api_key, resource_id)
    measurement_response = obj._post_data(endpoint, measurement_single)  # pylint: disable=protected-access

    # Assert
    assert isinstance(measurement_response, dict)
    assert measurement_response == json.loads(measurement_single)
    assert responses.calls[0].request.headers['Authorization'] == 'Basic MTIzNDU6'


@responses.activate
def test_post_data_timeout():
    """Test post data with timeout."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    uri = '/blah'
    expected_url = f'{BASE_URL}{uri}'

    measurement_single = """{
        "measurement": {
            "period_end": "2018-02-02T03:30:00.0000000Z",
            "period": "PT5M",
            "total_power": 1.23456
        }
    }"""

    responses.add(
        responses.POST,
        expected_url,
        body=ConnectTimeout(),
        status=200,
        content_type='applicaiton/json'
    )

    # Act
    obj = PySolcast(api_key, resource_id)
    with pytest.raises(ConnectTimeout):
        obj._post_data(uri, measurement_single)  # pylint: disable=protected-access

    # Assert
    assert responses.calls[0].request.headers['Authorization'] == 'Basic MTIzNDU6'



"""World Solar Radiation Module."""
import logging
from pysolcast.base import PySolcast


class World(PySolcast):
    """Class for interacting with world solar radiation endpoint.

    Refer to https://docs.solcast.com.au/#world-solar-radiation-data for more information.
    """

    base_uri = 'world_radiation'

    def __init__(self, api_key):  # pylint: disable=super-init-not-called
        self.api_key = api_key
        self.logger = logging.getLogger()

    def get_forecasts(self, latitude: str, longitude: str, hours: str = None) -> dict:
        """Get forecasts data for given location.

        :param latitude: The latitude of the location (EPSG:4326)
        :param longitude: The longitude of the location (EPSG:4326)
        :param hours: Time window of the response in hours
        :return: forecasts
        :raises ValidationError:
            Latitude, longitude or hours are invalid, see response_status for further details
        :raises SiteNotFound: The location is outside our coverage area.
        """
        endpoint = 'forecasts'
        payload = {
            'latitude': latitude,
            'longitude': longitude,
            'hours': hours
        }
        return self._get_data(self._create_uri(self.base_uri, endpoint), params=payload)

    def get_estimated_actuals(self, latitude: str, longitude: str, hours: str = None) -> dict:
        """Get estimated actuals data for given location.

        :param latitude: The latitude of the location (EPSG:4326)
        :param longitude: The longitude of the location (EPSG:4326)
        :param hours: Time window of the response in hours
        :return: estimated_actuals
        :raises ValidationError:
            Latitude, longitude or hours are invalid, see response_status for further details
        :raises SiteNotFound: The location is outside our coverage area.
        """
        endpoint = 'estimated_actuals'
        payload = {
            'latitude': latitude,
            'longitude': longitude,
            'hours': hours
        }
        return self._get_data(self._create_uri(self.base_uri, endpoint), params=payload)

    def _create_uri(self, uri: str, endpoint: str) -> str:
        """Create a URI for specific endpoint."""
        return f'/{uri}/{endpoint}'



"""Rooftop Site Module."""
from pysolcast.base import PySolcast, parse_date_time


class RooftopSite(PySolcast):
    """Class for interacting with Legacy Rooftop sites.

    Refer to https://docs.solcast.com.au/#58ca9bc0-27d4-4418-937f-03986331f01d for more information.
    """

    base_uri = 'rooftop_sites'

    def get_forecasts(self, params: dict = None) -> dict:
        """Get forecasts data for site.

        :return: forecasts:
        :raises ValidationError:
        :raises SiteError:
        """
        endpoint = 'forecasts'
        return self._get_data(self._create_uri(self.base_uri, endpoint), params)

    def get_forecasts_parsed(self, params: dict = None) -> dict:
        """Get forecasts data for site.

        :return: forecasts: Date is parsed as a datetime object.
        :raises ValidationError:
        :raises SiteError:
        """
        endpoint = 'forecasts'
        forecasts = self._get_data(self._create_uri(self.base_uri, endpoint), params)
        return parse_date_time(forecasts, endpoint)

    def get_estimated_actuals(self) -> dict:
        """Get estimated actuals data for site.

        :return: estimated_actuals:
        :raises ValidationError:
        :raises SiteError:
        """
        endpoint = 'estimated_actuals'
        return self._get_data(self._create_uri(self.base_uri, endpoint))

    def post_measurements(self, data: dict) -> dict:
        """Post measurement data for site.

        :param data: measurement or measurements
        :return: data: Returns data submitted
        :raises ValidationError:
        :raises SiteError:
        """
        endpoint = 'measurements'
        return self._post_data(self._create_uri(self.base_uri, endpoint), data)



"""Utility Site Module."""
from pysolcast.base import PySolcast


class UtilitySite(PySolcast):
    """Class for interacting with utility sites.

    Refer to https://docs.solcast.com.au/#utility-scale-sites for more information.
    """

    base_uri = 'utility_scale_sites'

    def get_forecasts(self, period: str, hours: str) -> dict:
        """Get forecasts data for site.

        :param period: Length of the averaging period in ISO8601 duration format.
        :param hours: An offset to which the number of forecasts will be included in the response.
        :return: forecasts
        :raises SiteError:
        """
        endpoint = 'forecasts'
        payload = {
            'Period': period,
            'Hours': hours
        }
        return self._get_data(self._create_uri(self.base_uri, endpoint), params=payload)

    def get_estimated_actuals(self, period: str, hours: str) -> dict:
        """Get estimated actuals data for site.

        :param period: Length of the averaging period in ISO8601 duration format.
        :param hours: An offset to which the number of forecasts will be included in the response.
        :return: estimated_actuals
        :raises SiteError:
        """
        endpoint = 'estimated_actuals'
        payload = {
            'Period': period,
            'Hours': hours
        }
        return self._get_data(self._create_uri(self.base_uri, endpoint), params=payload)

    def get_radiation_forecasts(self, period: str, hours: str) -> dict:
        """Get radiation forecasts data for site.

        :param period: Length of the averaging period in ISO8601 duration format.
        :param hours: An offset to which the number of forecasts will be included in the response.
        :return: forecasts
        :raises SiteError:
        """
        endpoint = 'weather/forecasts'
        payload = {
            'Period': period,
            'Hours': hours
        }
        return self._get_data(self._create_uri(self.base_uri, endpoint), params=payload)

    def get_radiation_estimated_actuals(self, period: str, hours: str) -> dict:
        """Get radiation estimated actual data for site.

        :param period: Length of the averaging period in ISO8601 duration format.
        :param hours: An offset to which the number of forecasts will be included in the response.
        :return: estimated_actuals
        :raises SiteError:
        """
        endpoint = 'weather/estimated_actuals'
        payload = {
            'Period': period,
            'Hours': hours
        }
        return self._get_data(self._create_uri(self.base_uri, endpoint), params=payload)

    def post_measurements(self, data: dict) -> dict:
        """Post measurement data for site.

        :param data: measurement or measurements
        :return: data: data submitted
        :raises ValidationError:
        :raises SiteError:
        """
        endpoint = 'measurements'
        return self._post_data(self._create_uri(self.base_uri, endpoint), data)



"""PySolcast Base Module.

Extended by site classes.
"""
import logging
from isodate import parse_datetime, parse_duration
from requests import get, post
import requests.exceptions
from pysolcast.exceptions import SiteError, ValidationError, RateLimitExceeded


class PySolcast:  # pylint: disable=too-few-public-methods
    """PySolcast class."""

    base_url = 'https://api.solcast.com.au'

    def __init__(self, api_key: str, resource_id: str):
        self.api_key = api_key
        self.resource_id = resource_id
        self.logger = logging.getLogger()

    def _get_data(self, uri: str, params: dict = None, timeout=60) -> dict:  # pylint: disable=inconsistent-return-statements
        """Get data from API."""
        url = f'{PySolcast.base_url}{uri}'
        payload = {'format': 'json'}
        if params:
            payload = {**payload, **params}
        try:
            _get_response = get(url, auth=(self.api_key, ''), params=payload, timeout=timeout)
        except (requests.exceptions.ConnectionError, requests.exceptions.Timeout) as error:
            self.logger.info(f'Error getting data: {error}')  # pylint: disable=logging-fstring-interpolation
            raise error
        if _get_response.status_code == 200:
            return _get_response.json()
        if _get_response.status_code == 429:
            self.logger.info('Solcast API rate limit reached.')
            self.logger.info('headers: %s', _get_response.headers)
            self.logger.info('text: %s', _get_response.text)
            raise RateLimitExceeded(
                f"Rate limit exceeded. Reset time: {_get_response.headers.get('x-rate-limit-reset')}")  # pylint: disable=line-too-long
        if _get_response.status_code == 400:
            self.logger.info('Validation error: %s', _get_response.headers)
            raise ValidationError('Validation error')
        if _get_response.status_code == 404:
            self.logger.info('Site error: %s', _get_response.headers)
            raise SiteError('Site error')

    def _post_data(self, uri: str, data: dict, timeout=60) -> dict:  # pylint: disable=inconsistent-return-statements
        """Post data to API."""
        url = f'{PySolcast.base_url}{uri}'
        try:
            _post_response = post(url, json=data, auth=(self.api_key, ''), timeout=timeout)
        except (requests.exceptions.ConnectionError, requests.exceptions.Timeout) as error:
            self.logger.info(error)
            raise error
        if _post_response.status_code == 200:
            return _post_response.json()
        if _post_response.status_code == 400:
            raise ValidationError
        if _post_response.status_code == 404:
            raise SiteError

    def _create_uri(self, uri: str, endpoint: str) -> str:
        """Create a URI for specific endpoint."""
        return f'/{uri}/{self.resource_id}/{endpoint}'


def parse_date_time(dic: dict, tld_key: str) -> dict:
    """Parse datetime and duration objects."""
    for item in dic[tld_key]:
        for key, value in item.items():
            if key == 'period_end':
                item[key] = parse_datetime(value)
            if key == 'period':
                item[key] = parse_duration(value)
    return dic



"""Tests for `UtilitySite` class."""

import responses
import pytest
from pysolcast.utility import UtilitySite
from pysolcast.exceptions import ValidationError, SiteError, RateLimitExceeded

BASE_URL = 'https://api.solcast.com.au'
UTILTY_URI = 'utility_scale_sites'


def test_UtilitySite():
    """Test creating object."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'

    # Act
    site = UtilitySite(api_key, resource_id)

    # Assert
    assert isinstance(site, UtilitySite)


@responses.activate
def test_get_forecasts_200():
    """Test get_forecasts with 200 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'forecasts'
    period = 'PT30M'
    hours = '170'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'

    forecast_response = {
        "forecasts": [
            {
                "pv_estimate": "9.5",
                "pv_estimate10": "6",
                "pv_estimate90": "13.8",
                "period_end": "2018-01-01T01:00:00.00000Z",
                "period": "PT30M"
            },
            {
                "pv_estimate": "10",
                "pv_estimate10": "8",
                "pv_estimate90": "12",
                "period_end": "2018-01-01T12:30:00.00000Z",
                "period": "PT30M"
            }
        ]
    }

    responses.add(
        responses.GET,
        expected_url,
        json=forecast_response,
        status=200,
        content_type='applicaiton/json'
    )

    # Act
    site = UtilitySite(api_key, resource_id)
    forecasts = site.get_forecasts(period, hours)

    # Assert
    assert isinstance(forecasts, dict)
    assert len(forecasts['forecasts']) == 2


@responses.activate
def test_get_forecasts_400():
    """Test get_forecasts with 400 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'forecasts'
    period = 'PT30M'
    hours = '170'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'

    responses.add(
        responses.GET, expected_url, status=400
    )

    # Act
    site = UtilitySite(api_key, resource_id)
    with pytest.raises(ValidationError):
        site.get_forecasts(period, hours)

    # Assert


@responses.activate
def test_get_forecasts_404():
    """Test get_forecasts with 400 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'forecasts'
    period = 'PT30M'
    hours = '170'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'

    responses.add(
        responses.GET, expected_url, status=404
    )

    # Act
    site = UtilitySite(api_key, resource_id)
    with pytest.raises(SiteError):
        site.get_forecasts(period, hours)

    # Assert


@responses.activate
def test_get_forecasts_429():
    """Test get_forecasts with 429 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'forecasts'
    period = 'PT30M'
    hours = '170'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'
    headers = {
        'x-rate-limit-reset': '155555555'
    }

    responses.add(
        responses.GET, expected_url, status=429, adding_headers=headers)

    # Act
    site = UtilitySite(api_key, resource_id)
    with pytest.raises(RateLimitExceeded):
        site.get_forecasts(period, hours)

    # Assert


@responses.activate
def test_get_estimated_actuals_200():
    """Test get_estimated_actuals with 200 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'estimated_actuals'
    period = 'PT30M'
    hours = '170'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'

    estimated_actual_response = {
        "estimated_actuals": [
            {
                "pv_estimate": "10",
                "period_end": "2018-01-01T01:00:00.00000Z",
                "period": "PT30M"
            },
            {
                "pv_estimate": "9",
                "period_end": "2018-01-01T12:30:00.00000Z",
                "period": "PT30M"
            }
        ]
    }

    responses.add(
        responses.GET,
        expected_url,
        json=estimated_actual_response,
        status=200,
        content_type='applicaiton/json'
    )

    # Act
    site = UtilitySite(api_key, resource_id)
    estimated_actuals = site.get_estimated_actuals(period, hours)

    # Assert
    assert isinstance(estimated_actuals, dict)
    assert len(estimated_actuals['estimated_actuals']) == 2


@responses.activate
def test_get_estimated_actuals_400():
    """Test get_estimated_actuals with 400 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'estimated_actuals'
    period = 'PT30M'
    hours = '170'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'

    responses.add(
        responses.GET, expected_url, status=400
    )

    # Act
    site = UtilitySite(api_key, resource_id)
    with pytest.raises(ValidationError):
        site.get_estimated_actuals(period, hours)

    # Assert


@responses.activate
def test_get_estimated_actuals_404():
    """Test get_estimated_actuals with 404 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'estimated_actuals'
    period = 'PT30M'
    hours = '170'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'

    responses.add(
        responses.GET, expected_url, status=404
    )

    # Act
    site = UtilitySite(api_key, resource_id)
    with pytest.raises(SiteError):
        site.get_estimated_actuals(period, hours)

    # Assert


@responses.activate
def test_get_estimated_actuals_429():
    """Test get_estimated_actuals with 429 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'estimated_actuals'
    period = 'PT30M'
    hours = '170'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'
    headers = {
        'x-rate-limit-reset': '155555555'
    }
    responses.add(
        responses.GET, expected_url, status=429, adding_headers=headers
    )

    # Act
    site = UtilitySite(api_key, resource_id)
    with pytest.raises(RateLimitExceeded):
        site.get_estimated_actuals(period, hours)

    # Assert


@responses.activate
def test_get_radiation_forecasts_200():
    """Test get_radiation_forecasts with 200 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'weather/forecasts'
    period = 'PT30M'
    hours = '170'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'

    forecast_response = {
        "forecasts": [
            {
                "pv_estimate": "9.5",
                "pv_estimate10": "6",
                "pv_estimate90": "13.8",
                "period_end": "2018-01-01T01:00:00.00000Z",
                "period": "PT30M"
            },
            {
                "pv_estimate": "10",
                "pv_estimate10": "8",
                "pv_estimate90": "12",
                "period_end": "2018-01-01T12:30:00.00000Z",
                "period": "PT30M"
            }
        ]
    }

    responses.add(
        responses.GET,
        expected_url,
        json=forecast_response,
        status=200,
        content_type='applicaiton/json'
    )

    # Act
    site = UtilitySite(api_key, resource_id)
    forecasts = site.get_radiation_forecasts(period, hours)

    # Assert
    assert isinstance(forecasts, dict)
    assert len(forecasts['forecasts']) == 2


@responses.activate
def test_get_radiation_forecasts_400():
    """Test get_radiation_forecasts with 400 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'weather/forecasts'
    period = 'PT30M'
    hours = '170'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'

    responses.add(
        responses.GET, expected_url, status=400
    )

    # Act
    site = UtilitySite(api_key, resource_id)
    with pytest.raises(ValidationError):
        site.get_radiation_forecasts(period, hours)

    # Assert


@responses.activate
def test_get_radiation_forecasts_404():
    """Test get_radiation_forecasts with 400 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'weather/forecasts'
    period = 'PT30M'
    hours = '170'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'

    responses.add(
        responses.GET, expected_url, status=404
    )

    # Act
    site = UtilitySite(api_key, resource_id)
    with pytest.raises(SiteError):
        site.get_radiation_forecasts(period, hours)

    # Assert


@responses.activate
def test_get_radiation_forecasts_429():
    """Test get_radiation_forecasts with 429 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'weather/forecasts'
    period = 'PT30M'
    hours = '170'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'
    headers = {
        'x-rate-limit-reset': '155555555'
    }

    responses.add(
        responses.GET, expected_url, status=429, adding_headers=headers)

    # Act
    site = UtilitySite(api_key, resource_id)
    with pytest.raises(RateLimitExceeded):
        site.get_radiation_forecasts(period, hours)

    # Assert


@responses.activate
def test_get_radiation_estimated_actuals_200():
    """Test get_radiation_estimated_actuals with 200 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'weather/estimated_actuals'
    period = 'PT30M'
    hours = '170'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'

    estimated_actual_response = {
        "estimated_actuals": [
            {
                "pv_estimate": "10",
                "period_end": "2018-01-01T01:00:00.00000Z",
                "period": "PT30M"
            },
            {
                "pv_estimate": "9",
                "period_end": "2018-01-01T12:30:00.00000Z",
                "period": "PT30M"
            }
        ]
    }

    responses.add(
        responses.GET,
        expected_url,
        json=estimated_actual_response,
        status=200,
        content_type='applicaiton/json'
    )

    # Act
    site = UtilitySite(api_key, resource_id)
    estimated_actuals = site.get_radiation_estimated_actuals(period, hours)

    # Assert
    assert isinstance(estimated_actuals, dict)
    assert len(estimated_actuals['estimated_actuals']) == 2


@responses.activate
def test_get_radiation_estimated_actuals_400():
    """Test get_radiation_estimated_actuals with 400 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'weather/estimated_actuals'
    period = 'PT30M'
    hours = '170'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'

    responses.add(
        responses.GET, expected_url, status=400
    )

    # Act
    site = UtilitySite(api_key, resource_id)
    with pytest.raises(ValidationError):
        site.get_radiation_estimated_actuals(period, hours)

    # Assert


@responses.activate
def test_get_radiation_estimated_actuals_404():
    """Test get_radiation_estimated_actuals with 404 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'weather/estimated_actuals'
    period = 'PT30M'
    hours = '170'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'

    responses.add(
        responses.GET, expected_url, status=404
    )

    # Act
    site = UtilitySite(api_key, resource_id)
    with pytest.raises(SiteError):
        site.get_radiation_estimated_actuals(period, hours)

    # Assert


@responses.activate
def test_get_radiation_estimated_actuals_429():
    """Test get_radiation_estimated_actuals with 429 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'weather/estimated_actuals'
    period = 'PT30M'
    hours = '170'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'
    headers = {
        'x-rate-limit-reset': '155555555'
    }
    responses.add(
        responses.GET, expected_url, status=429, adding_headers=headers
    )

    # Act
    site = UtilitySite(api_key, resource_id)
    with pytest.raises(RateLimitExceeded):
        site.get_radiation_estimated_actuals(period, hours)

    # Assert


@responses.activate
def test_post_measurements_single():
    """Test post_measurements with 200 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'measurements'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'

    measurement_single = {
        "measurement": {
            "period_end": "2018-02-02T03:30:00.0000000Z",
            "period": "PT5M",
            "total_power": 1.23456
        }
    }

    responses.add(
        responses.POST,
        expected_url,
        json=measurement_single,
        status=200,
        content_type='applicaiton/json'
    )

    # Act
    site = UtilitySite(api_key, resource_id)
    measurement_response = site.post_measurements(measurement_single)

    # Assert
    assert isinstance(measurement_response, dict)
    assert measurement_response == measurement_single


@responses.activate
def test_post_measurements_400():
    """Test post_measurements with 400 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'measurements'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'
    measurement_single = {
        "measurement": {
            "period_end": "2018-02-02T03:30:00.0000000Z",
            "period": "PT5M",
            "total_power": 1.23456
        }
    }

    responses.add(
        responses.POST, expected_url, status=400
    )

    # Act
    site = UtilitySite(api_key, resource_id)
    with pytest.raises(ValidationError):
        site.post_measurements(measurement_single)

    # Assert


@responses.activate
def test_post_measurements_404():
    """Test post_measurements with 404 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'measurements'
    expected_url = f'{BASE_URL}/{UTILTY_URI}/{resource_id}/{endpoint}'
    measurement_single = {
        "measurement": {
            "period_end": "2018-02-02T03:30:00.0000000Z",
            "period": "PT5M",
            "total_power": 1.23456
        }
    }

    responses.add(
        responses.POST, expected_url, status=404
    )

    # Act
    site = UtilitySite(api_key, resource_id)
    with pytest.raises(SiteError):
        site.post_measurements(measurement_single)

    # Assert



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = []

test_requirements = ['pytest>=3', ]


setup(
    author="Nathaniel McAuliffe",
    author_email='nathanielmcauliffe@hotmail.com',
    python_requires='>=3.8',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    description="Solcast API",
    install_requires=requirements,
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='solcast',
    name='pysolcast',
    packages=find_packages(include=['pysolcast', 'pysolcast.*']),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/mcaulifn/solcast',
    zip_safe=False,
)



"""Tests for `WeatherSite` class."""

import responses
import pytest
from pysolcast.weather import WeatherSite
from pysolcast.exceptions import ValidationError, SiteError, RateLimitExceeded

BASE_URL = 'https://api.solcast.com.au'
WEATHER_URI = 'weather_sites'


def test_RooftopSite():
    """Test creating object."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'

    # Act
    site = WeatherSite(api_key, resource_id)

    # Assert
    assert isinstance(site, WeatherSite)


@responses.activate
def test_get_forecasts_200():
    """Test get_forecasts with 200 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'forecasts'
    expected_url = f'{BASE_URL}/{WEATHER_URI}/{resource_id}/{endpoint}'

    forecast_response = {
        "forecasts": [
            {
                "ghi": "10",
                "ghi90": "12",
                "ghi10": "8",
                "ebh": "10",
                "dni": "10",
                "dni90": "12",
                "dni10": "8",
                "air_temp": "20",
                "zenith": "90",
                "azimuth": "45.1234",
                "cloud_opacity": "12",
                "period_end": "2018-01-01T01:00:00.00000Z",
                "period": "PT30"
            },
            {
                "ghi": "10",
                "ghi90": "12",
                "ghi10": "8",
                "ebh": "10",
                "dni": "10",
                "dni90": "12",
                "dni10": "8",
                "air_temp": "20",
                "zenith": "90",
                "azimuth": "45.1234",
                "cloud_opacity": "12",
                "period_end": "2018-01-01T12:30:00.00000Z",
                "period": "PT30"
            }
        ]
    }

    responses.add(
        responses.GET,
        expected_url,
        json=forecast_response,
        status=200,
        content_type='applicaiton/json'
    )

    # Act
    site = WeatherSite(api_key, resource_id)
    forecasts = site.get_forecasts()

    # Assert
    assert isinstance(forecasts, dict)
    assert len(forecasts['forecasts']) == 2


@responses.activate
def test_get_forecasts_400():
    """Test get_forecasts with 400 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'forecasts'
    expected_url = f'{BASE_URL}/{WEATHER_URI}/{resource_id}/{endpoint}'

    responses.add(
        responses.GET, expected_url, status=400
    )

    # Act
    site = WeatherSite(api_key, resource_id)
    with pytest.raises(ValidationError):
        site.get_forecasts()

    # Assert


@responses.activate
def test_get_forecasts_404():
    """Test get_forecasts with 400 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'forecasts'
    expected_url = f'{BASE_URL}/{WEATHER_URI}/{resource_id}/{endpoint}'

    responses.add(
        responses.GET, expected_url, status=404
    )

    # Act
    site = WeatherSite(api_key, resource_id)
    with pytest.raises(SiteError):
        site.get_forecasts()

    # Assert


@responses.activate
def test_get_forecasts_429():
    """Test get_forecasts with 429 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'forecasts'
    expected_url = f'{BASE_URL}/{WEATHER_URI}/{resource_id}/{endpoint}'
    headers = {
        'x-rate-limit-reset': '155555555'
    }

    responses.add(
        responses.GET, expected_url, status=429, adding_headers=headers)

    # Act
    site = WeatherSite(api_key, resource_id)
    with pytest.raises(RateLimitExceeded):
        site.get_forecasts()

    # Assert


@responses.activate
def test_get_estimated_actuals_200():
    """Test get_estimated_actuals with 200 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'estimated_actuals'
    expected_url = f'{BASE_URL}/{WEATHER_URI}/{resource_id}/{endpoint}'

    estimated_actual_response = {
        "estimated_actuals": [
            {
                "ghi": "10",
                "ebh": "10",
                "dni": "10",
                "dhi": "10",
                "cloud_opacity": "12",
                "period_end": "2018-01-01T01:05:00.00000Z",
                "period": "PT5M"
            },
            {
                "ghi": "10",
                "ebh": "10",
                "dni": "10",
                "dhi": "10",
                "cloud_opacity": "12",
                "period_end": "2018-01-01T01:00:00.00000Z",
                "period": "PT5M"
            }
        ]
    }

    responses.add(
        responses.GET,
        expected_url,
        json=estimated_actual_response,
        status=200,
        content_type='applicaiton/json'
    )

    # Act
    site = WeatherSite(api_key, resource_id)
    estimated_actuals = site.get_estimated_actuals()

    # Assert
    assert isinstance(estimated_actuals, dict)
    assert len(estimated_actuals['estimated_actuals']) == 2


@responses.activate
def test_get_estimated_actuals_400():
    """Test get_estimated_actuals with 400 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'estimated_actuals'
    expected_url = f'{BASE_URL}/{WEATHER_URI}/{resource_id}/{endpoint}'

    responses.add(
        responses.GET, expected_url, status=400
    )

    # Act
    site = WeatherSite(api_key, resource_id)
    with pytest.raises(ValidationError):
        site.get_estimated_actuals()

    # Assert


@responses.activate
def test_get_estimated_actuals_404():
    """Test get_estimated_actuals with 404 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'estimated_actuals'
    expected_url = f'{BASE_URL}/{WEATHER_URI}/{resource_id}/{endpoint}'

    responses.add(
        responses.GET, expected_url, status=404
    )

    # Act
    site = WeatherSite(api_key, resource_id)
    with pytest.raises(SiteError):
        site.get_estimated_actuals()

    # Assert


@responses.activate
def test_get_estimated_actuals_429():
    """Test get_estimated_actuals with 429 status code."""
    # Arrange
    api_key = '12345'
    resource_id = '1234-1234'
    endpoint = 'estimated_actuals'
    expected_url = f'{BASE_URL}/{WEATHER_URI}/{resource_id}/{endpoint}'
    headers = {
        'x-rate-limit-reset': '155555555'
    }
    responses.add(
        responses.GET, expected_url, status=429, adding_headers=headers
    )

    # Act
    site = WeatherSite(api_key, resource_id)
    with pytest.raises(RateLimitExceeded):
        site.get_estimated_actuals()

    # Assert
